#pragma BEGINDUMP

#include <windows.h>
#include "hbapi.h"

#ifdef Use_Extendet
   #define _WIN32_WINNT   0x0400        // dupe
   #ifdef __XHARBOUR__
   #define HB_STORNI( n, x, y ) hb_storni( n, x, y )
   #else
   #define HB_STORNI( n, x, y ) hb_storvni( n, x, y )
   #endif
#else
   #include <mgdefs.h>
#endif

#define NIL                        (0)  // Nothing...
//  Drawing styles

#define CENTER                      0
#define TILE                        1
#define STRETCH                     2

HB_FUNC ( DRAWPICTURE )
{
    HWND       hWnd = ( HWND ) hb_parnl( 1 );
    HDC        dc = GetDC( hWnd );
    HANDLE     picture;
    BITMAP     bitmap;
    HDC        bits;
    HANDLE     old;

    POINT      size;
    POINT      origin = { NIL,NIL };

    RECT       box;
    int        row;
    int        col;

    int desktopx = GetSystemMetrics (SM_CXSCREEN);
    int desktopy = GetSystemMetrics (SM_CYSCREEN);

    if ((picture = LoadImage (NIL,hb_parc(2),IMAGE_BITMAP,NIL,NIL,LR_LOADFROMFILE)) == NULL) {
        hb_retl (FALSE);
        }

    if ((bits = CreateCompatibleDC (dc)) == NULL) {
        DeleteObject (picture);
        hb_retl (FALSE);
        }

    if ((old = SelectObject (bits,picture)) == NULL) {
        DeleteObject (picture);
        DeleteDC (bits);
        hb_retl (FALSE);
        }

    SetMapMode (bits,GetMapMode (dc));

    if (!GetObject (picture,sizeof (BITMAP), (LPSTR) &bitmap)) {
        SelectObject (bits,old);
        DeleteObject (picture);
        DeleteDC (bits);
        hb_retl (FALSE);
        }

    size.x = bitmap.bmWidth;
    size.y = bitmap.bmHeight;
    DPtoLP (dc,&size,1);

    origin.x = NIL;
    origin.y = NIL;
    DPtoLP (bits,&origin,1);

    switch (hb_parnl(3)) {

        case CENTER :

             box.left = (desktopx - size.x) / 2;
             box.top  = (desktopy - size.y) / 2;

             BitBlt (dc,
                     box.left,
                     box.top,
                     size.x,
                     size.y,
                     bits,
                     origin.x,
                     origin.y,
                     SRCCOPY);
             break;

        case TILE :

             for (row = NIL; row < ((desktopy / size.y) + 1); row++) {

                 for (col = NIL; col < ((desktopx / size.x) + 1); col++) {

                     box.left = col * size.x;
                     box.top = row * size.y;

                     BitBlt (dc,
                             box.left,
                             box.top,
                             size.x,
                             size.y,
                             bits,
                             origin.x,
                             origin.y,
                             SRCCOPY);
                     }
                 }
              break;

         case STRETCH :

              StretchBlt (dc,
                          NIL,
                          NIL,
                          desktopx,
                          desktopy,
                          bits,
                          origin.x,
                          origin.y,
                          size.x,
                          size.y,
                          SRCCOPY);
              break;
              }

    SelectObject (bits,old);
    DeleteDC     (bits);
    DeleteObject (picture);
    ReleaseDC(hWnd, dc);

    hb_retl (TRUE);
}

HB_FUNC ( C_PUTFILE2 )
{
    OPENFILENAME ofn;
    char buffer[512];

    int flags = OFN_FILEMUSTEXIST | OFN_EXPLORER ;

    if ( hb_parl(4) )
    {
        flags = flags | OFN_NOCHANGEDIR ;
    }

    strcpy( buffer, "" );

    memset( (void*) &ofn, 0, sizeof( OPENFILENAME ) );
    ofn.lStructSize = sizeof(ofn);
    ofn.hwndOwner = GetActiveWindow() ;
    ofn.lpstrFilter = hb_parc(1) ;
    ofn.lpstrFile = buffer;
    ofn.nMaxFile = 512;
    ofn.lpstrInitialDir = hb_parc(3);
    ofn.nFilterIndex    = hb_parni(5);
    ofn.lpstrTitle = hb_parc(2) ;
    ofn.Flags = flags;

    if( GetSaveFileName( &ofn ) )
    {
        hb_stornl(ofn.nFilterIndex, 5 );
        hb_retc( ofn.lpstrFile );
    }
    else
    {
        hb_retc( "" );
    }
}

HB_FUNC( BEGINPAINT )
{
   HWND hWnd = ( HWND ) ( LONG_PTR ) HB_PARNL( 1 );

   if( IsWindow( hWnd ) )
   {
      PAINTSTRUCT ps;

      HB_RETNL( ( LONG_PTR ) BeginPaint( hWnd, &ps ) );

      hb_storclen( ( const char * ) &ps, sizeof( PAINTSTRUCT ), 2 );
   }
   else
      HB_RETNL( ( LONG_PTR ) NULL );
}

HB_FUNC( ENDPAINT )
{
   HWND hWnd         = ( HWND ) ( LONG_PTR ) HB_PARNL( 1 );
   PAINTSTRUCT * pps = ( PAINTSTRUCT * ) hb_parc( 2 );

   if( IsWindow( hWnd ) && pps )
      hb_retl( EndPaint( hWnd, pps ) );
   else
      hb_retl( HB_FALSE );
}



HB_FUNC( GETDESKTOPAREA )
{
   RECT rect;

   SystemParametersInfo( SPI_GETWORKAREA, 1, &rect, 0 );

   hb_reta( 4 );
   HB_STORNI( ( INT ) rect.left, -1, 1 );
   HB_STORNI( ( INT ) rect.top, -1, 2 );
   HB_STORNI( ( INT ) rect.right, -1, 3 );
   HB_STORNI( ( INT ) rect.bottom, -1, 4 );
}


#include "CORE.C"

#pragma ENDDUMP
